import time

from nose.tools import *

import colmet.metrics.procstats_default as td
from colmet.metrics.base import get_counters_class

def test_get_counters_class():
    '''Test: import the procstats_default metric class'''
    Counter = get_counters_class('procstats_default')
    assert(Counter._get_metric_name() == 'procstats_default')


def setup_metric():
    global counter
    global timestamp

    Counter = get_counters_class('procstats_default')
    counter = Counter()

def test_counters_initialisation():
    '''Test: procstats_metric initialization'''
    setup_metric()




'''
Colmet-node User Interface
'''
import logging
import argparse
import signal
import sys

from colmet import VERSION
from colmet.common.backends.zeromq import ZMQInputBackend
from colmet.common.backends.base import StdoutBackend
from colmet.common.utils import Daemon
from colmet.common.exceptions import Error, NoneValueError


LOG = logging.getLogger()


class Task(object):

    def __init__(self, name, options):
        self.name = name
        self.options = options
        self.output_backends = []
        if self.options.hdf5_filepath is not None:
            from colmet.collector.hdf5 import HDF5OutputBackend
            self.output_backends.append(HDF5OutputBackend(self.options))
        if self.options.enable_stdout_backend:
            self.output_backends.append(StdoutBackend(self.options))
        self.zeromq_input_backend = ZMQInputBackend(self.options)
        self.counters_list = []
        self.buffer_size = self.options.buffer_size

    def start(self):
        LOG.info("Starting %s" % self.name)
        for backend in self.output_backends:
            backend.open()
        self.zeromq_input_backend.open()
        self.zeromq_input_backend.on_recv(self.collect)
        signal.signal(signal.SIGINT, self.terminate)
        signal.signal(signal.SIGTERM, self.terminate)
        self.zeromq_input_backend.loop.start()

    def push(self):
        for backend in self.output_backends:
            try:
                backend.push(self.counters_list)
                LOG.info("%s new metrics has been pushed with %s"
                         % (len(self.counters_list),
                            backend.get_backend_name()))
            except (NoneValueError, TypeError):
                LOG.debug("Values for metrics are not there.")
        del self.counters_list[:]

    def collect(self, counter):
        self.counters_list.append(counter)
        if len(self.counters_list) >= self.options.buffer_size:
            self.push()

    def terminate(self, *args, **kwargs):
        LOG.info("Terminating %s" % self.name)
        self.zeromq_input_backend.loop.stop()
        self.zeromq_input_backend.close()
        self.push()
        for backend in self.output_backends:
            backend.close()
        sys.exit(0)


#
# Main program
#

DESCRIPTION = '''Display and/or collect cpu, memory and i/o bandwidth used
by the processes in a cpuset or a cgroup.'''


def main():
    formatter = argparse.ArgumentDefaultsHelpFormatter
    parser = argparse.ArgumentParser(description=DESCRIPTION,
                                     formatter_class=formatter)

    parser.add_argument('--version', action='version',
                        version='colmet version %s' % VERSION)

    parser.add_argument('-v', '--verbose', action='count', dest="verbosity",
                        default=1)

    parser.add_argument('--daemon', dest='run_as_daemon',
                        help='Runs as daemon',
                        action='store_true', default=False)

    parser.add_argument('--pidfile', dest='pidfile',
                        default="/var/run/colmet.pid",
                        help='Pid file used when running as daemon')

    parser.add_argument('--logfile', dest='logfile',
                        default="/var/log/colmet.log",
                        help='Logger file used when running as daemon')

    parser.add_argument('--buffer-size', dest='buffer_size', default=100,
                        help='Defines the maximum number of counters that '
                             'colmet should queue in memory before pushing '
                             'it to output backend', type=int)

    parser.add_argument("--enable-stdout-backend", action='store_true',
                        help='Prints the metrics on STDOUT',
                        dest='enable_stdout_backend', default=False)

    group = parser.add_argument_group('Zeromq')

    group.add_argument("--zeromq-bind-uri", dest='zeromq_bind_uri',
                       help="ZeroMQ bind URI",
                       default='tcp://0.0.0.0:5556')
    group.add_argument("--zeromq-hwm", type=int,
                       default=1000, dest='zeromq_hwm',
                       help="The high water mark is a hard limit on the"
                            " maximum number of outstanding messages "
                            " ZeroMQ shall queue in memory. The value "
                            " of zero means \"no limit\".")

    group.add_argument("--zeromq-swap", type=int, dest='zeromq_swap',
                       default=(200 * 2 ** 10),
                       help="Defines the maximum size of the swap space in"
                            " bytes. ZeroMQ will use a this local swapfile to"
                            " store messages that exceed the high water mark.")

    group.add_argument("--zeromq-linger", type=int,
                       default=0, dest='zeromq_linger',
                       help="Set the linger period for the specified socket."
                            " The value of -1 specifies an infinite linger"
                            " period. The value of 0 specifies no linger"
                            " period.  Positive values specify an upper bound"
                            " for the  linger period in milliseconds.")

    group = parser.add_argument_group('HDF5')

    group.add_argument("--hdf5-filepath", dest='hdf5_filepath', default=None,
                       help='The file path used to store the hdf5 data')

    group.add_argument("--hdf5-complevel", type=int, dest='hdf5_complevel',
                       help='Specifies a compression level for data. '
                            'The allowed range is 0-9. A value of 0 '
                            '(the default) disables compression.',
                       default=0)

    group.add_argument("--hdf5-complib", dest='hdf5_complib', default='zlib',
                       help='Specifies the compression library to be used. '
                            '"zlib" (the default), "lzo", "bzip2" and "blosc" '
                            'are supported.')

    args = parser.parse_args()

    if (args.hdf5_filepath is None and args.enable_stdout_backend is False):
        parser.error("You need to provide at least one output backend "
                     "[hdf5|stdout]")

    # Set the logging value (always display CRITICAL and ERROR)
    logging.basicConfig(
        format='%(asctime)s - %(levelname)s - %(message)s',
        datefmt='%d/%m/%Y %H:%M:%S',
        level=40 - args.verbosity * 10,
    )

    # run
    main_loop = lambda: Task(sys.argv[0], args).start()
    try:
        if not args.run_as_daemon:
            main_loop()
        else:
            daemon = Daemon(args.pidfile, main_loop, stderr=args.logfile)
            daemon.start()
    except KeyboardInterrupt:
        pass
    except Error as err:
        err.show()
        sys.exit(1)
    except Exception as err:
        MSG = "Error not handled '%s'" % err.__class__.__name__
        logging.critical(MSG)
        if logging.getLogger().getEffectiveLevel() <= logging.DEBUG:
            print repr(err)
        raise


if __name__ == '__main__':
    main()




from nose.tools import *

from colmet.importer import get_module
from colmet.exceptions import UnableToImportModuleError

@raises(UnableToImportModuleError)
def test_if_importer_fail_if_module_not_found():
    get_module('colmet.metrics','this_module_does_not_exist')

def test_if_importer_success_to_import_module():
    m=get_module('colmet.metrics','taskstats_default')
    assert(m.__name__ == "colmet.metrics.taskstats_default")




"""
Testing the base metric class
"""
from nose.tools import *

from colmet.metrics.base import *

def test_basic_types():
    assert(UInt8().length == 2)
    assert(UInt16().length == 2)
    assert(UInt32().length == 4)
    assert(UInt64().length == 8)
    for i in [1,5,0,100]:
        assert( String(i).length==i)

def test_basecounters_packing_unpacking():
    import os
    import ctypes

    raw = os.urandom(BaseCounters._fmt_length)

    m = BaseCounters()

    m.unpack_from(raw,0)

    raw2=ctypes.create_string_buffer(BaseCounters._fmt_length)

    m.pack_into(raw2,0)

    assert raw2.raw == raw


class MyCounters(BaseCounters):
    _headers = [
        ('project',String(16),'string')
    ]

    _counters = [
        ('speed',UInt64(),'count','add','Speed')
    ]

m = None

def setup_mycounters():
    global m
    m = MyCounters()
    m.job_id = 42
    m.timestamp = 0
    m.hostname = 'localhost'

def teardown_mycounters():
    global m
    del m

def test_register_header():
    '''Testing registering header'''
    assert 'project' in  MyCounters._header_definitions

    (h_type,h_repr,h_index,h_offset) = MyCounters._header_definitions['project']

    assert h_type == String(16)
    assert h_repr == 'string'
    assert h_offset != None
    assert h_index != None

def test_register_counters():
    '''Testing registering counter'''
    assert 'speed' in  MyCounters._counter_definitions

    class MyCounters2(MyCounters):

        _counters = [
            ('speed2',UInt32(),'count','add','Speed2')
        ]

    assert 'speed2' in  MyCounters2._counter_definitions

    (c_type,c_repr,c_acc_fn, c_descr, c_index,c_offset) = MyCounters2._counter_definitions['speed2']

    assert c_type == UInt32()
    assert c_repr == 'count'
    assert c_acc_fn == 'add'
    assert c_descr == 'Speed2'
    assert c_offset != None
    assert c_index != None

@with_setup(setup_mycounters,teardown_mycounters)
def test_counter_assignment():
    '''Testing basic counters get/set'''
    m.speed = 120
    assert(m.speed == 120)
    assert(m._get_counter('speed') == 120)

@with_setup(setup_mycounters,teardown_mycounters)
def test_counter_packed_assignment():
    '''Testing packed/unpacked counters get/set'''
    m.project = 'speedcontrol'
    m.speed = 0
    m.pack()
    assert(m.speed == 0)
    m.speed = 120
    assert(m.speed == 120)
    m.unpack()
    assert(m.speed == 120)

@raises(CounterAlreadyExistError)
def test_failure_if_regitering_an_existing_counter():
    '''Registering an existing counter should fail'''
    class MyCounters2(MyCounters):
        _counters = [
            ('speed',UInt64(),'count','add','Speed')
        ]

@raises(CounterAlreadyExistError)
def test_failure_if_regitering_an_existing_headerr():
    '''Registering an existing counter should fail'''
    class MyCounters2(MyCounters):
        _headers = [
            ('job_id',UInt64(),'string')
        ]


def test_normalize():
    '''Testing formating value (Normalization)'''
    val = BaseCounters._normalize(1,1000,10000,['','K','G'])
    assert val == "1"
    val = BaseCounters._normalize(1000,1000,10000,['','K','G'])
    assert val == "1000"
    val = BaseCounters._normalize(100000,1000,10000,['','K','G'])
    assert val == "100K"

@raises(TypeError)
def test_failure_if_deriving_counters_from_multiple_class():
    class MyCounters2(MyCounters):
        pass

    class MyCounters3(MyCounters, MyCounters2):
        pass



#!/usr/bin/env python
# *-* coding: utf-8 *-*

# Copyright 2012 Philippe Le Brouster <philippe.le-brouster@imag.fr> / LIG
#
# This file is part of Colmet
#
# Colmet is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Épeire is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero Public License for more details.
#
# You should have received a copy of the GNU Affero Public License
# along with Épeire.  If not, see <http://www.gnu.org/licenses/>.


'''
Colmet Python setup file
'''

import os.path as op
from setuptools import setup, find_packages
from colmet import VERSION

here = op.abspath(op.dirname(__file__))


def read(fname):
    ''' Return the file content. '''
    with open(op.join(here, fname)) as f:
        return f.read()


setup(
    name="colmet",
    version=VERSION.strip('-dev'),
    description=("A utility to monitor the jobs ressources in a HPC"
                 " environment, espacially OAR"),
    keywords="monitoring, taskstat, oar, hpc, sciences",
    maintainer='Salem Harrache',
    maintainer_email='salem.harrache@inria.fr',
    author="Philippe Le Brouster, Olivier Richard",
    author_email="philippe.le-brouster@imag.fr, olivier.richard@imag.fr",
    url="http://oar.imag.fr/",
    packages=find_packages(),
    long_description=read('README.rst') + '\n\n' + read('CHANGELOG.rst'),
    install_requires=read('requirements.txt').splitlines(),
    platforms=['Linux'],
    license="GNU GPL",
    tests_require=['nose>=1.0'],
    test_suite='nose.collector',
    entry_points={
        'console_scripts': [
            'colmet-node = colmet.node.main:main',
            'colmet-collector = colmet.collector.main:main',
        ],
    },
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: System :: Monitoring',
        'Topic :: System :: Clustering',
        'Programming Language :: Python :: 2.5',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7'
    ]
)



'''
Colmet-node User Interface
'''
import logging
import optparse
import signal
import sys
import time

from colmet import VERSION
from colmet.node.backends.procstats import ProcstatsBackend
from colmet.node.backends.taskstats import TaskstatsBackend
from colmet.common.backends.zeromq import ZMQOutputBackend
from colmet.common.utils import AsyncFileNotifier, as_thread
from colmet.common.utils import Daemon, optparse_add_default_value
from colmet.common.exceptions import Error, NoneValueError


LOG = logging.getLogger()


class Task(object):

    def __init__(self, options):
        self.options = options
        self.taskstats_backend = TaskstatsBackend(self.options)
        self.procstats_backend = ProcstatsBackend(self.options)
        self.zeromq_output_backend = ZMQOutputBackend(self.options)

    @as_thread
    def check_jobs_thread(self):
        notifier = \
            AsyncFileNotifier(paths=self.options.cpuset_rootpath,
                              callback=self.taskstats_backend.update_job_list)
        # Initial job list update
        self.update_job_list()
        notifier.loop()

    def update_job_list(self):
        self.taskstats_backend.update_job_list()

    def start(self):
        LOG.info("Starting Colmet-node")
        self.taskstats_backend.open()
        self.procstats_backend.open()
        self.zeromq_output_backend.open()
        signal.signal(signal.SIGINT, self.terminate)
        signal.signal(signal.SIGTERM, self.terminate)
        self.check_jobs_thread.start()
        self.loop()

    def terminate(self, *args, **kwargs):
        LOG.info("Terminating Colmet-node")
        self.zeromq_output_backend.close()
        sys.exit(0)

    def sleep(self):
        now = time.time()
        time_towait = (((now // self.options.sampling_period) + 1) *
                       self.options.sampling_period - now)
        time.sleep(time_towait)

    def loop(self):
        while True:
            now = time.time()
            LOG.debug("Gathering the metrics")
            counters_list = []
            for backend in [self.taskstats_backend, self.procstats_backend]:
                pulled_counters = backend.pull()

                if backend.get_backend_name() == 'taskstats':
                    if len(pulled_counters) > 0:
                        for counters in pulled_counters:
                            counters_list += counters
                else:
                    counters_list += pulled_counters

                LOG.debug("%s metrics has been pulled width %s" %
                          (len(pulled_counters), backend.get_backend_name()))

            LOG.debug("time to take measure: %s sec" % (time.time() - now))

            if len(counters_list) > 0:
                #print "nb counters_list", len(counters_list)
                try:
                    self.zeromq_output_backend.push(counters_list)
                    LOG.debug("%s metrics has been pushed with zeromq"
                              % (len(counters_list)))
                except (NoneValueError, TypeError):
                    LOG.debug("Values for metrics are not there.")
            # sleep to next sampling
            self.sleep()
        LOG.info("Ending Colmet")


#
# Main program
#

USAGE = '''%s [OPTIONS]

Collect cpu, memory and i/o bandwidth used by the processes in a cpuset or a
cgroup and send metrics from zeromq''' % sys.argv[0]


def main():
    parser = optparse.OptionParser(usage=USAGE, version='colmet ' + VERSION)

    parser.add_option('-v', '--verbose', action='count', dest="verbosity",
                      default=0)

    parser.add_option('-s', '--sample-period', type='float',
                      dest='sampling_period', metavar='SEC', default=5,
                      help='sampling period of measuring [5 second]')

    parser.add_option('--daemon', dest='run_as_daemon',
                      help='Run as daemon [False]',
                      action='store_true', default=False)

    parser.add_option('--pidfile', dest='pidfile', type="str",
                      default="/var/run/colmet.pid",
                      help='pid file when running as daemon [/var/run/colmet.pid]')

    parser.add_option('--logfile', dest='logfile', type="str",
                      default="/var/log/colmet.log",
                      help='pid file when running as daemon [/var/log/colmet.log]')

    group = optparse.OptionGroup(parser, "Taskstat")

    group.add_option('-c', '--cgroup', type='str', dest='cgroups',
                     action='append', default=[],
                     help='cgroup/cpuset to monitor', metavar='CGROUP')
    group.add_option('-p', '--pid', type='int', dest='pids',
                     action='append', default=[],
                     help='process id to monitor', metavar='PID')
    group.add_option('-t', '--tid', type='int', dest='tids',
                     action='append', default=[],
                     help='task ids to monitor', metavar='TID')
    group.add_option('--cpuset_rootpath', type='str', dest='cpuset_rootpath',
                     action='append', default=["/dev/oar_cgroups/oar/"],
                     help='cpuset root path', metavar='CPUSETROOTPATH')
    group.add_option('--regex_job_id', type='str', dest='regex_job_id',
                     action='append', default=['_(\d+)$'],
                     metavar='REGEXJOBID',
                     help='regular expression to extract job_id from '
                          'cpuset directory name', )
    parser.add_option_group(group)

    group = optparse.OptionGroup(parser, "Zeromq")
    group.add_option("--zeromq-uri", type='str',
                     dest='zeromq_uri', default='tcp://127.0.0.1:5556')
    group.add_option("--zeromq-hwm", type='int', dest='zeromq_hwm',
                     default=1000,
                     help="The high water mark is a hard limit on the maximum"
                          " number of outstanding messages ZeroMQ shall queue"
                          " in memory. The value of zero means \"no limit\".")
    group.add_option("--zeromq-swap", type='int', dest='zeromq_swap',
                     default=(200 * 2 ** 10),
                     help="Defines the maximum size of the swap space in"
                          " bytes. ZeroMQ will use a this local swapfile to"
                          " store messages that exceed the high water mark.")
    group.add_option("--zeromq-linger", type='int',
                     default=0, dest='zeromq_linger',
                     help="Set the linger period for the specified socket."
                          " The value of -1 specifies an infinite linger"
                          " period. The value of 0 specifies no linger period."
                          " Positive values specify an upper bound for the "
                          " linger period in milliseconds.")

    parser.add_option_group(group)

    optparse_add_default_value(parser)

    options, args = parser.parse_args()
    if args:
        parser.error('Unexpected arguments: ' + ' '.join(args))

    # Set the logging value (always display CRITICAL and ERROR)
    logging.basicConfig(
        format='%(asctime)s - %(levelname)s - %(message)s',
        datefmt='%d/%m/%Y %H:%M:%S',
        level=40 - options.verbosity * 10,
    )

    # run
    main_loop = lambda: Task(options).start()
    try:
        if not options.run_as_daemon:
            main_loop()
        else:
            daemon = Daemon(options.pidfile, main_loop, stderr=options.logfile)
            daemon.start()
    except KeyboardInterrupt:
        pass
    except Error as err:
        err.show()
        sys.exit(1)
    except Exception as err:
        MSG = "Error not handled '%s'" % err.__class__.__name__
        logging.critical(MSG)
        if logging.getLogger().getEffectiveLevel() <= logging.DEBUG:
            print repr(err)
        raise


if __name__ == '__main__':
    main()



import asyncore
import atexit
import logging
import os
import signal
import sys
from threading import Thread
import time

from pyinotify import ProcessEvent, AsyncNotifier, WatchManager
import pyinotify

LOG = logging.getLogger()


class as_thread(object):
    """Decorator that make any method a thread object."""
    def __init__(self, func, name=None, doc=None):
        self.__name__ = name or func.__name__
        self.__module__ = func.__module__
        self.__doc__ = doc or func.__doc__
        self.func = func

    def __get__(self, obj, type=None):
        if obj is None:
            return self
        _thread = obj.__dict__.get(self.__name__)
        if _thread is None:
            _thread = Thread(target=self.func, args=(obj,))
            # thread.daemon causes the thread to terminate when the main
            # process ends
            _thread.daemon = True
            obj.__dict__[self.__name__] = _thread
        return _thread


def wait_dir(path):
    """Wait until the `path` folder creation."""
    while True:
        if os.path.exists(path) and os.path.isdir(path):
            break
        time.sleep(5)


class AsyncFileNotifier(object):
    def __init__(self, paths, callback):
        class EventHandler(ProcessEvent):
            def process_IN_CREATE(self, event):
                callback()

            def process_IN_DELETE(self, event):
                callback()

        manager = WatchManager()  # Watch Manager
        mask = pyinotify.IN_DELETE | pyinotify.IN_CREATE  # watched events
        self.async_notifier = AsyncNotifier(manager, EventHandler())
        # aggregate inotify events
        self.async_notifier.coalesce_events()
        for path in paths:
            if os.path.exists(path):
                manager.add_watch(path, mask, rec=False)
            else:
                LOG.warn("%s folder doesn't exist yet." % path)
                wait_dir(path)
                manager.add_watch(path, mask, rec=False)
                LOG.info("%s has just been created." % path)

    def loop(self):
        LOG.info("Starting inotify daemon.")
        asyncore.loop()

# ***
# Modified generic daemon class
# ***

# Author:         http://www.jejik.com/articles/2007/02/
#                         a_simple_unix_linux_daemon_in_python/www.boxedice.com

# License:        http://creativecommons.org/licenses/by-sa/3.0/

# Changes:        23rd Jan 2009 (David Mytton <david@boxedice.com>)
#                 - Replaced hard coded '/dev/null in __init__ with os.devnull
#                 - Added OS check to conditionally remove code that doesn't
#                   work on OS X
#                 - Added output to console on completion
#                 - Tidied up formatting
#                 11th Mar 2009 (David Mytton <david@boxedice.com>)
#                 - Fixed problem with daemon exiting on Python 2.4
#                   (before SystemExit was part of the Exception base)
#                 13th Aug 2010 (David Mytton <david@boxedice.com>
#                 - Fixed unhandled exception if PID file is empty
# Core modules


class Daemon(object):
    """
    A generic daemon class.

    Usage: subclass the Daemon class and override the run() method
    """
    def __init__(self, pidfile, main, stdin=os.devnull,
                 stdout=os.devnull, stderr=os.devnull,
                 home_dir='.', umask=022):
        self.stdin = stdin
        self.stdout = stdout
        self.stderr = stderr
        self.pidfile = pidfile
        self.home_dir = home_dir
        self.umask = umask
        self.main = main
        self.daemon_alive = True

    def daemonize(self):
        """
        Do the UNIX double-fork magic, see Stevens' "Advanced
        Programming in the UNIX Environment" for details (ISBN 0201563177)
        http://www.erlenstar.demon.co.uk/unix/faq_2.html#SEC16
        """
        try:
            pid = os.fork()
            if pid > 0:
                # Exit first parent
                sys.exit(0)
        except OSError, e:
            sys.stderr.write(
                "fork #1 failed: %d (%s)\n" % (e.errno, e.strerror))
            sys.exit(1)

        # Decouple from parent environment
        os.chdir(self.home_dir)
        os.setsid()
        os.umask(self.umask)

        # Do second fork
        try:
            pid = os.fork()
            if pid > 0:
                # Exit from second parent
                sys.exit(0)
        except OSError, e:
            sys.stderr.write(
                "fork #2 failed: %d (%s)\n" % (e.errno, e.strerror))
            sys.exit(1)

        if sys.platform != 'darwin':  # This block breaks on OS X
            # Redirect standard file descriptors
            sys.stdout.flush()
            sys.stderr.flush()
            si = file(self.stdin, 'r')
            so = file(self.stdout, 'a+')
            if self.stderr:
                se = file(self.stderr, 'a+', 0)
            else:
                se = so
            os.dup2(si.fileno(), sys.stdin.fileno())
            os.dup2(so.fileno(), sys.stdout.fileno())
            os.dup2(se.fileno(), sys.stderr.fileno())

        def sigtermhandler(signum, frame):
            self.daemon_alive = False
            signal.signal(signal.SIGTERM, sigtermhandler)
            signal.signal(signal.SIGINT, sigtermhandler)

        # Write pidfile
        atexit.register(
            self.delpid)  # Make sure pid file is removed if we quit
        pid = str(os.getpid())
        file(self.pidfile, 'w+').write("%s\n" % pid)

    def delpid(self):
        os.remove(self.pidfile)

    def start(self, *args, **kwargs):
        """
        Start the daemon
        """
        # Check for a pidfile to see if the daemon already runs
        try:
            pf = file(self.pidfile, 'r')
            pid = int(pf.read().strip())
            pf.close()
        except IOError:
            pid = None
        except SystemExit:
            pid = None

        if pid:
            message = "pidfile %s already exists. Is it already running?\n"
            sys.stderr.write(message % self.pidfile)
            sys.exit(1)

        # Start the daemon
        self.daemonize()
        self.run(*args, **kwargs)

    def run(self):
        """
        You should override this method when you subclass Daemon.
        It will be called after the process has been
        daemonized by start() or restart().
        """
        self.main()
